# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import datetime

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields

import djstripe.fields


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('djstripe', '0008_auto_20150806_1641'),
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(verbose_name='created',
                                                                default=django.utils.timezone.now, editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(verbose_name='modified',
                                                                      default=django.utils.timezone.now,
                                                                      editable=False)),
                ('stripe_id', djstripe.fields.StripeIdField(max_length=50, unique=True)),
                ('livemode', djstripe.fields.StripeNullBooleanField(
                    help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                    Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                    operation.', default=False)),
                ('stripe_timestamp', djstripe.fields.StripeDateTimeField(
                    help_text='The datetime this object was created in stripe.', null=True)),
                ('metadata', djstripe.fields.StripeJSONField(
                    blank=True, help_text='A set of key/value pairs that you can attach to an object. It can be useful\
                     for storing additional information about an object in a structured format.', null=True)),
                ('description', djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.',
                                                                null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='StripeSource',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(verbose_name='created',
                                                                default=django.utils.timezone.now, editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(verbose_name='modified',
                                                                      default=django.utils.timezone.now,
                                                                      editable=False)),
                ('stripe_id', djstripe.fields.StripeIdField(max_length=50, unique=True)),
                ('livemode', djstripe.fields.StripeNullBooleanField(
                    help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                    Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                    operation.', default=False)),
                ('stripe_timestamp', djstripe.fields.StripeDateTimeField(
                    help_text='The datetime this object was created in stripe.', null=True)),
                ('metadata', djstripe.fields.StripeJSONField(
                    blank=True, help_text='A set of key/value pairs that you can attach to an object. It can be useful\
                     for storing additional information about an object in a structured format.', null=True)),
                ('description', djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.',
                                                                null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Card',
            fields=[
                ('stripesource_ptr', models.OneToOneField(serialize=False, parent_link=True, primary_key=True, on_delete=django.db.models.deletion.CASCADE,
                                                          to='djstripe.StripeSource', auto_created=True)),
                ('address_city', djstripe.fields.StripeTextField(help_text='Billing address city.', null=True)),
                ('address_country', djstripe.fields.StripeTextField(help_text='Billing address country.', null=True)),
                ('address_line1', djstripe.fields.StripeTextField(help_text='Billing address (Line 1).', null=True)),
                ('address_line1_check', djstripe.fields.StripeCharField(
                    choices=[('pass', 'Pass'), ('fail', 'Fail'), ('unavailable', 'Unavailable'),
                             ('unknown', 'Unknown')],
                    max_length=11, help_text='If ``address_line1`` was provided, results of the check.', null=True)),
                ('address_line2', djstripe.fields.StripeTextField(help_text='Billing address (Line 2).', null=True)),
                ('address_state', djstripe.fields.StripeTextField(help_text='Billing address state.', null=True)),
                ('address_zip', djstripe.fields.StripeTextField(help_text='Billing address zip code.', null=True)),
                ('address_zip_check', djstripe.fields.StripeCharField(
                    choices=[('pass', 'Pass'), ('fail', 'Fail'), ('unavailable', 'Unavailable'),
                             ('unknown', 'Unknown')],
                    max_length=11, help_text='If ``address_zip`` was provided, results of the check.', null=True)),
                ('brand', djstripe.fields.StripeCharField(
                    choices=[('Visa', 'Visa'), ('American Express', 'American Express'), ('MasterCard', 'MasterCard'),
                             ('Discover', 'Discover'), ('JCB', 'JCB'), ('Diners Club', 'Diners Club'),
                             ('Unknown', 'Unknown')],
                    max_length=16, help_text='Card brand.')),
                ('country', djstripe.fields.StripeCharField(
                    max_length=2, help_text='Two-letter ISO code representing the country of the card.')),
                ('cvc_check', djstripe.fields.StripeCharField(
                    choices=[('pass', 'Pass'), ('fail', 'Fail'), ('unavailable', 'Unavailable'),
                             ('unknown', 'Unknown')],
                    max_length=11, help_text='If a CVC was provided, results of the check.', null=True)),
                ('dynamic_last4', djstripe.fields.StripeCharField(
                    max_length=4,
                    help_text='(For tokenized numbers only.) The last four digits of the device account number.',
                    null=True)),
                ('exp_month', djstripe.fields.StripeIntegerField(help_text='Card expiration month.')),
                ('exp_year', djstripe.fields.StripeIntegerField(help_text='Card expiration year.')),
                ('fingerprint', djstripe.fields.StripeTextField(
                    help_text='Uniquely identifies this particular card number.', null=True)),
                ('funding', djstripe.fields.StripeCharField(
                    choices=[('credit', 'Credit'), ('debit', 'Debit'), ('prepaid', 'Prepaid'), ('unknown', 'Unknown')],
                    max_length=7, help_text='Card funding type.')),
                ('last4', djstripe.fields.StripeCharField(max_length=4, help_text='Last four digits of Card number.')),
                ('name', djstripe.fields.StripeTextField(help_text='Cardholder name.', null=True)),
                ('tokenization_method', djstripe.fields.StripeCharField(
                    choices=[('apple_pay', 'Apple Pay'), ('android_pay', 'Android Pay')], max_length=11,
                    help_text='If the card number is tokenized, this is the method that was used.', null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('djstripe.stripesource',),
        ),
        migrations.RemoveField(
            model_name='transferchargefee',
            name='transfer',
        ),
        migrations.RemoveField(
            model_name='charge',
            name='card_kind',
        ),
        migrations.RemoveField(
            model_name='charge',
            name='card_last_4',
        ),
        migrations.RemoveField(
            model_name='charge',
            name='invoice',
        ),
        migrations.RemoveField(
            model_name='customer',
            name='card_exp_month',
        ),
        migrations.RemoveField(
            model_name='customer',
            name='card_exp_year',
        ),
        migrations.RemoveField(
            model_name='customer',
            name='card_fingerprint',
        ),
        migrations.RemoveField(
            model_name='customer',
            name='card_kind',
        ),
        migrations.RemoveField(
            model_name='customer',
            name='card_last_4',
        ),
        migrations.RemoveField(
            model_name='event',
            name='validated_message',
        ),
        migrations.RemoveField(
            model_name='invoiceitem',
            name='line_type',
        ),
        migrations.RemoveField(
            model_name='subscription',
            name='amount',
        ),
        migrations.RemoveField(
            model_name='transfer',
            name='event',
        ),
        migrations.AddField(
            model_name='charge',
            name='account',
            field=models.ForeignKey(
                related_name='charges', on_delete=django.db.models.deletion.CASCADE, null=True, to='djstripe.Account',
                help_text='The account the charge was made on behalf of. Null here indicates that this value was \
                never set.'),
        ),
        migrations.AddField(
            model_name='charge',
            name='source',
            field=models.ForeignKey(related_name='charges', on_delete=django.db.models.deletion.CASCADE, null=True, to='djstripe.StripeSource'),
        ),
        migrations.AddField(
            model_name='charge',
            name='currency',
            field=djstripe.fields.StripeCharField(help_text='Three-letter ISO currency code representing the currency \
            in which the charge was made.', max_length=3, default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='charge',
            name='failure_code',
            field=djstripe.fields.StripeCharField(
                choices=[('invalid_number', 'Invalid Number'), ('invalid_expiry_month', 'Invalid Expiry Month'),
                         ('invalid_expiry_year', 'Invalid Expiry Year'), ('invalid_cvc', 'Invalid Cvc'),
                         ('incorrect_number', 'Incorrect Number'), ('expired_card', 'Expired Card'),
                         ('incorrect_cvc', 'Incorrect Cvc'), ('incorrect_zip', 'Incorrect Zip'),
                         ('card_declined', 'Card Declined'), ('missing', 'Missing'),
                         ('processing_error', 'Processing Error'), ('rate_limit', 'Rate Limit')],
                max_length=30, help_text='Error code explaining reason for charge failure if available.', null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='failure_message',
            field=djstripe.fields.StripeTextField(
                help_text='Message to user further explaining reason for charge failure if available.', null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='fee_details',
            field=djstripe.fields.StripeJSONField(null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='fraudulent',
            field=djstripe.fields.StripeBooleanField(
                help_text='Whether or not this charge was marked as fraudulent.', default=False),
        ),
        migrations.AddField(
            model_name='charge',
            name='livemode',
            field=djstripe.fields.StripeNullBooleanField(
                help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                operation.', default=False),
        ),
        migrations.AddField(
            model_name='charge',
            name='metadata',
            field=djstripe.fields.StripeJSONField(
                blank=True, help_text='A set of key/value pairs that you can attach to an object. It can be useful for\
                 storing additional information about an object in a structured format.',
                null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='shipping',
            field=djstripe.fields.StripeJSONField(help_text='Shipping information for the charge', null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='source_stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, help_text='The payment source id.', null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='source_type',
            field=djstripe.fields.StripeCharField(
                max_length=20, help_text='The payment source type. If the payment source is supported by dj-stripe, \
                a corresponding model is attached to this Charge via a foreign key matching this field.', null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='statement_descriptor',
            field=djstripe.fields.StripeCharField(
                max_length=22, help_text='An arbitrary string to be displayed on your customer\'s credit card \
                statement. The statement description may not include <>"\' characters, and will appear on your \
                customer\'s statement in capital letters. Non-ASCII characters are automatically stripped. While \
                most banks display this information consistently, some may display it incorrectly or not at all.',
                null=True),
        ),
        migrations.AddField(
            model_name='charge',
            name='status',
            field=djstripe.fields.StripeCharField(help_text='The status of the payment.',
                                                  choices=[('succeeded', 'Succeeded'), ('failed', 'Failed')],
                                                  max_length=10, default='unknown'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='charge',
            name='transfer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='djstripe.Transfer',
                                    help_text='The transfer to the destination account (only applicable if the \
                                    charge was created using the destination parameter).'),
        ),
        migrations.AddField(
            model_name='customer',
            name='account_balance',
            field=djstripe.fields.StripeIntegerField(help_text="Current balance, if any, being stored on the \
            customer's account. If negative, the customer has credit to apply to the next invoice. If positive, the \
            customer has an amount owed that will be added to the next invoice. The balance does not refer to any \
            unpaid invoices; it solely takes into account amounts that have yet to be successfully applied to any \
            invoice. This balance is only taken into account for recurring charges.", null=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='business_vat_id',
            field=djstripe.fields.StripeCharField(max_length=20, help_text="The customer's VAT identification number.",
                                                  null=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='currency',
            field=djstripe.fields.StripeCharField(
                help_text='The currency the customer can be charged in for recurring billing purposes \
                (subscriptions, invoices, invoice items).', max_length=3, null=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='delinquent',
            field=djstripe.fields.StripeBooleanField(
                help_text="Whether or not the latest charge for the customer's latest invoice has failed.",
                default=False),
        ),
        migrations.AddField(
            model_name='customer',
            name='description',
            field=djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.', null=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='livemode',
            field=djstripe.fields.StripeNullBooleanField(
                help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                operation.', default=False),
        ),
        migrations.AddField(
            model_name='customer',
            name='metadata',
            field=djstripe.fields.StripeJSONField(
                blank=True,
                help_text='A set of key/value pairs that you can attach to an object. It can be useful for storing \
                additional information about an object in a structured format.', null=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='shipping',
            field=djstripe.fields.StripeJSONField(help_text='Shipping information associated with the customer.',
                                                  null=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='stripe_timestamp',
            field=djstripe.fields.StripeDateTimeField(help_text='The datetime this object was created in stripe.',
                                                      null=True),
        ),
        migrations.AddField(
            model_name='event',
            name='description',
            field=djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.', null=True),
        ),
        migrations.AddField(
            model_name='event',
            name='metadata',
            field=djstripe.fields.StripeJSONField(
                blank=True,
                help_text='A set of key/value pairs that you can attach to an object. It can be useful for storing \
                additional information about an object in a structured format.', null=True),
        ),
        migrations.AddField(
            model_name='event',
            name='received_api_version',
            field=djstripe.fields.StripeCharField(
                max_length=15,
                help_text='the API version at which the event data was rendered. Blank for old entries only, all \
                new entries will have this value', blank=True),
        ),
        migrations.AddField(
            model_name='event',
            name='request_id',
            field=djstripe.fields.StripeCharField(
                max_length=50,
                help_text="Information about the request that triggered this event, for traceability purposes. \
                If empty string then this is an old entry without that data. If Null then this is not an old entry, \
                but a Stripe 'automated' event with no associated request.", blank=True, null=True),
        ),
        migrations.AddField(
            model_name='event',
            name='stripe_timestamp',
            field=djstripe.fields.StripeDateTimeField(
                help_text='The datetime this object was created in stripe.', null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='amount_due',
            field=djstripe.fields.StripeCurrencyField(
                help_text="Final amount due at this time for this invoice. If the invoice's total is smaller than \
                the minimum charge amount, for example, or if there is account credit that can be applied to the \
                invoice, the amount_due may be 0. If there is a positive starting_balance for the invoice \
                (the customer owes money), the amount_due will also take that into account. The charge that gets \
                generated for the invoice will be for the amount specified in amount_due.",
                max_digits=7, default=0, decimal_places=2),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='invoice',
            name='application_fee',
            field=djstripe.fields.StripeCurrencyField(
                max_digits=7,
                help_text="The fee in cents that will be applied to the invoice and transferred to the application \
                owner's Stripe account when the invoice is paid.", decimal_places=2, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='currency',
            field=djstripe.fields.StripeCharField(help_text='Three-letter ISO currency code.',
                                                  max_length=3, default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='invoice',
            name='description',
            field=djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.', null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='ending_balance',
            field=djstripe.fields.StripeIntegerField(
                help_text='Ending customer balance after attempting to pay invoice. If the invoice has not been \
                attempted yet, this will be null.', null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='forgiven',
            field=djstripe.fields.StripeBooleanField(
                help_text='Whether or not the invoice has been forgiven. Forgiving an invoice instructs us to \
                update the subscription status as if the invoice were successfully paid. Once an invoice has been \
                forgiven, it cannot be unforgiven or reopened.', default=False),
        ),
        migrations.AddField(
            model_name='invoice',
            name='livemode',
            field=djstripe.fields.StripeNullBooleanField(
                help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                operation.', default=False),
        ),
        migrations.AddField(
            model_name='invoice',
            name='metadata',
            field=djstripe.fields.StripeJSONField(
                blank=True,
                help_text='A set of key/value pairs that you can attach to an object. It can be useful for storing \
                additional information about an object in a structured format.',
                null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='next_payment_attempt',
            field=djstripe.fields.StripeDateTimeField(help_text='The time at which payment will next be attempted.',
                                                      null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='starting_balance',
            field=djstripe.fields.StripeIntegerField(
                help_text='Starting customer balance before attempting to pay invoice. If the invoice has not been \
                attempted yet, this will be the current customer balance.', default=0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='invoice',
            name='statement_descriptor',
            field=djstripe.fields.StripeCharField(
                max_length=22,
                help_text='An arbitrary string to be displayed on your customer\'s credit card statement. \
                The statement description may not include <>"\' characters, and will appear on your customer\'s \
                statement in capital letters. Non-ASCII characters are automatically stripped. While most banks \
                display this information consistently, some may display it incorrectly or not at all.', null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='stripe_timestamp',
            field=djstripe.fields.StripeDateTimeField(help_text='The datetime this object was created in stripe.',
                                                      null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='subscription',
            field=models.ForeignKey(
                related_name='invoices',
                on_delete=django.db.models.deletion.CASCADE,
                null=True,
                to='djstripe.Subscription',
                help_text='The subscription that this invoice was prepared for, if any.'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='subscription_proration_date',
            field=djstripe.fields.StripeDateTimeField(
                help_text='Only set for upcoming invoices that preview prorations. The time used to calculate \
                prorations.',
                null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='tax',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, help_text='The amount of tax included in the \
            total, calculated from ``tax_percent`` and the subtotal. If no ``tax_percent`` is defined, this value \
            will be null.', decimal_places=2, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='tax_percent',
            field=djstripe.fields.StripePercentField(
                validators=[django.core.validators.MinValueValidator(1.0),
                            django.core.validators.MaxValueValidator(100.0)],
                max_digits=5,
                help_text="This percentage of the subtotal has been added to the total amount of the invoice, \
                including invoice line items and discounts. This field is inherited from the subscription's \
                ``tax_percent`` field, but can be changed before the invoice is paid. This field defaults to null.",
                decimal_places=2, null=True),
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='customer',
            field=models.ForeignKey(related_name='invoiceitems', on_delete=django.db.models.deletion.CASCADE, default=1, to='djstripe.Customer',
                                    help_text='The customer associated with this invoiceitem.'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='date',
            field=djstripe.fields.StripeDateTimeField(help_text='The date on the invoiceitem.',
                                                      default=datetime.datetime(2100, 1, 1, 0, 0,
                                                                                tzinfo=django.utils.timezone.utc)),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='discountable',
            field=djstripe.fields.StripeBooleanField(help_text='If True, discounts will apply to this invoice item. \
            Always False for prorations.', default=False),
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='livemode',
            field=djstripe.fields.StripeNullBooleanField(
                help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                operation.', default=False),
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='metadata',
            field=djstripe.fields.StripeJSONField(
                blank=True,
                help_text='A set of key/value pairs that you can attach to an object. It can be useful for storing \
                additional information about an object in a structured format.', null=True),
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='stripe_timestamp',
            field=djstripe.fields.StripeDateTimeField(help_text='The datetime this object was created in stripe.',
                                                      null=True),
        ),
        migrations.AddField(
            model_name='invoiceitem',
            name='subscription',
            field=models.ForeignKey(
                related_name='invoiceitems',
                on_delete=django.db.models.deletion.CASCADE,
                null=True,
                to='djstripe.Subscription',
                help_text='The subscription that this invoice item has been created for, if any.'),
        ),
        migrations.AddField(
            model_name='plan',
            name='description',
            field=djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.', null=True),
        ),
        migrations.AddField(
            model_name='plan',
            name='livemode',
            field=djstripe.fields.StripeNullBooleanField(
                help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                operation.', default=False),
        ),
        migrations.AddField(
            model_name='plan',
            name='metadata',
            field=djstripe.fields.StripeJSONField(
                blank=True,
                help_text='A set of key/value pairs that you can attach to an object. It can be useful for storing \
                additional information about an object in a structured format.', null=True),
        ),
        migrations.AddField(
            model_name='plan',
            name='statement_descriptor',
            field=djstripe.fields.StripeCharField(
                max_length=22,
                help_text='An arbitrary string to be displayed on your customer\'s credit card statement. The \
                statement description may not include <>"\' characters, and will appear on your customer\'s statement \
                in capital letters. Non-ASCII characters are automatically stripped. While most banks display this \
                information consistently, some may display it incorrectly or not at all.', null=True),
        ),
        migrations.AddField(
            model_name='plan',
            name='stripe_timestamp',
            field=djstripe.fields.StripeDateTimeField(help_text='The datetime this object was created in stripe.',
                                                      null=True),
        ),
        migrations.AlterModelOptions(
            name='plan',
            options={'ordering': ['amount']},
        ),
        migrations.AddField(
            model_name='subscription',
            name='application_fee_percent',
            field=djstripe.fields.StripePercentField(
                validators=[django.core.validators.MinValueValidator(1.0),
                            django.core.validators.MaxValueValidator(100.0)],
                help_text='A positive decimal that represents the fee percentage of the subscription invoice amount \
                that will be transferred to the application owner’s Stripe account each billing period.',
                max_digits=5, decimal_places=2, null=True),
        ),
        migrations.AddField(
            model_name='subscription',
            name='description',
            field=djstripe.fields.StripeTextField(blank=True, help_text='A description of this object.', null=True),
        ),
        migrations.AddField(
            model_name='subscription',
            name='livemode',
            field=djstripe.fields.StripeNullBooleanField(
                help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                operation.', default=False),
        ),
        migrations.AddField(
            model_name='subscription',
            name='metadata',
            field=djstripe.fields.StripeJSONField(
                blank=True,
                help_text='A set of key/value pairs that you can attach to an object. It can be useful for storing \
                additional information about an object in a structured format.', null=True),
        ),
        migrations.AddField(
            model_name='subscription',
            name='stripe_id',
            field=djstripe.fields.StripeIdField(max_length=50, default='unknown', unique=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='subscription',
            name='stripe_timestamp',
            field=djstripe.fields.StripeDateTimeField(help_text='The datetime this object was created in stripe.',
                                                      null=True),
        ),
        migrations.AddField(
            model_name='subscription',
            name='tax_percent',
            field=djstripe.fields.StripePercentField(
                validators=[django.core.validators.MinValueValidator(1.0),
                            django.core.validators.MaxValueValidator(100.0)],
                max_digits=5,
                help_text='A positive decimal (with at most two decimal places) between 1 and 100. This represents \
                the percentage of the subscription invoice subtotal that will be calculated and added as tax to the \
                final amount each billing period.', decimal_places=2, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='amount_reversed',
            field=djstripe.fields.StripeCurrencyField(
                max_digits=7,
                help_text='The amount reversed (can be less than the amount attribute on the transfer if a partial \
                reversal was issued).', decimal_places=2, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='currency',
            field=djstripe.fields.StripeCharField(help_text='Three-letter ISO currency code.',
                                                  max_length=3, default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='transfer',
            name='destination',
            field=djstripe.fields.StripeIdField(
                help_text='ID of the bank account, card, or Stripe account the transfer was sent to.',
                max_length=50, default='unknown'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='transfer',
            name='destination_payment',
            field=djstripe.fields.StripeIdField(
                max_length=50,
                help_text='If the destination is a Stripe account, this will be the ID of the payment that the \
                destination account received for the transfer.', null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='destination_type',
            field=djstripe.fields.StripeCharField(
                help_text='The type of the transfer destination.',
                choices=[('card', 'Card'), ('bank_account', 'Bank Account'), ('stripe_account', 'Stripe Account')],
                max_length=14, default='unknown'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='transfer',
            name='failure_code',
            field=djstripe.fields.StripeCharField(
                choices=[('insufficient_funds', 'Insufficient Funds'), ('account_closed', 'Account Closed'),
                         ('no_account', 'No Account'), ('invalid_account_number', 'Invalid Account Number'),
                         ('debit_not_authorized', 'Debit Not Authorized'),
                         ('bank_ownership_changed', 'Bank Ownership Changed'), ('account_frozen', 'Account Frozen'),
                         ('could_not_process', 'Could Not Process'),
                         ('bank_account_restricted', 'Bank Account Restricted'),
                         ('invalid_currency', 'Invalid Currency')],
                max_length=23,
                help_text='Error code explaining reason for transfer failure if available. See \
                https://stripe.com/docs/api/python#transfer_failures.', null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='failure_message',
            field=djstripe.fields.StripeTextField(
                help_text='Message to user further explaining reason for transfer failure if available.', null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='fee',
            field=djstripe.fields.StripeCurrencyField(max_digits=7, decimal_places=2, null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='fee_details',
            field=djstripe.fields.StripeJSONField(null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='livemode',
            field=djstripe.fields.StripeNullBooleanField(
                help_text='Null here indicates that the livemode status is unknown or was previously unrecorded. \
                Otherwise, this field indicates whether this record comes from Stripe test mode or live mode \
                operation.', default=False),
        ),
        migrations.AddField(
            model_name='transfer',
            name='metadata',
            field=djstripe.fields.StripeJSONField(
                blank=True,
                help_text='A set of key/value pairs that you can attach to an object. It can be useful for storing \
                additional information about an object in a structured format.', null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='reversed',
            field=djstripe.fields.StripeBooleanField(
                help_text='Whether or not the transfer has been fully reversed. If the transfer is only partially \
                reversed, this attribute will still be false.', default=False),
        ),
        migrations.AddField(
            model_name='transfer',
            name='source_transaction',
            field=djstripe.fields.StripeIdField(
                max_length=50,
                help_text='ID of the charge (or other transaction) that was used to fund the transfer. If null, the \
                transfer was funded from the available balance.', null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='source_type',
            field=djstripe.fields.StripeCharField(
                help_text='The source balance from which this transfer came.',
                choices=[('card', 'Card'), ('bank_account', 'Bank Account'), ('bitcoin_reciever', 'Bitcoin Reciever'),
                         ('alipay_account', 'Alipay Account')], max_length=16, default='unknown'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='transfer',
            name='statement_descriptor',
            field=djstripe.fields.StripeCharField(
                max_length=22,
                help_text='An arbitrary string to be displayed on your customer\'s credit card statement. The \
                statement description may not include <>"\' characters, and will appear on your customer\'s statement \
                in capital letters. Non-ASCII characters are automatically stripped. While most banks display this \
                information consistently, some may display it incorrectly or not at all.', null=True),
        ),
        migrations.AddField(
            model_name='transfer',
            name='stripe_timestamp',
            field=djstripe.fields.StripeDateTimeField(help_text='The datetime this object was created in stripe.',
                                                      null=True),
        ),
        migrations.AddField(
            model_name='stripesource',
            name='customer',
            field=models.ForeignKey(related_name='sources', on_delete=django.db.models.deletion.CASCADE, to='djstripe.Customer'),
        ),
        migrations.AddField(
            model_name='stripesource',
            name='polymorphic_ctype',
            field=models.ForeignKey(related_name='polymorphic_djstripe.stripesource_set+', on_delete=django.db.models.deletion.CASCADE,
                                    to='contenttypes.ContentType', editable=False, null=True),
        ),
        migrations.AddField(
            model_name='customer',
            name='default_source',
            field=models.ForeignKey(related_name='customers', on_delete=django.db.models.deletion.CASCADE, null=True, to='djstripe.StripeSource'),
        ),
        migrations.DeleteModel(
            name='TransferChargeFee',
        ),
    ]



# -*- coding: utf-8 -*-
"""
.. module:: dj-stripe.contrib.rest_framework.serializers.

    :synopsis: dj-stripe - Serializers to be used with the dj-stripe REST API.

.. moduleauthor:: Philippe Luickx (@philippeluickx)

"""

from __future__ import unicode_literals

from rest_framework import serializers
from rest_framework.serializers import ModelSerializer

from djstripe.models import Subscription


class SubscriptionSerializer(ModelSerializer):
    """A serializer used for the Subscription model."""

    class Meta(object):
        """Model class options."""

        model = Subscription


class CreateSubscriptionSerializer(serializers.Serializer):
    """A serializer used to create a Subscription."""

    stripe_token = serializers.CharField(max_length=200)
    plan = serializers.CharField(max_length=50)
    charge_immediately = serializers.NullBooleanField(required=False)
    tax_percent = serializers.DecimalField(
        required=False,
        max_digits=5,
        decimal_places=2,
    )



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.utils.

  :synopsis: dj-stripe - Utility functions related to the djstripe app.

.. moduleauthor:: @kavdev, @pydanny, @wahuneke
"""
import datetime

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import ImproperlyConfigured
from django.utils import timezone


ANONYMOUS_USER_ERROR_MSG = (
    "dj-stripe's payment checking mechanisms require the user "
    "be authenticated before use. Please use django.contrib.auth's "
    "login_required decorator or a LoginRequiredMixin. "
    "Please read the warning at "
    "http://dj-stripe.readthedocs.org/en/latest/usage.html#ongoing-subscriptions."
)


def subscriber_has_active_subscription(subscriber, plan=None):
    """
    Helper function to check if a subscriber has an active subscription.

    Throws improperlyConfigured if the subscriber is an instance of AUTH_USER_MODEL
    and get_user_model().is_anonymous == True.

    Activate subscription rules (or):
        * customer has active subscription

    If the subscriber is an instance of AUTH_USER_MODEL, active subscription rules (or):
        * customer has active subscription
        * user.is_superuser
        * user.is_staff

    :param subscriber: The subscriber for which to check for an active subscription.
    :type subscriber: dj-stripe subscriber
    :param plan: The plan for which to check for an active subscription. If plan is None and
                 there exists only one subscription, this method will check if that subscription
                 is active. Calling this method with no plan and multiple subscriptions will throw
                 an exception.
    :type plan: Plan or string (plan ID)

    """
    if isinstance(subscriber, AnonymousUser):
        raise ImproperlyConfigured(ANONYMOUS_USER_ERROR_MSG)

    if isinstance(subscriber, get_user_model()):
        if subscriber.is_superuser or subscriber.is_staff:
            return True
    from .models import Customer

    customer, created = Customer.get_or_create(subscriber)
    if created or not customer.has_active_subscription(plan):
        return False
    return True


def get_supported_currency_choices(api_key):
    """
    Pull a stripe account's supported currencies and returns a choices tuple of those supported currencies.

    :param api_key: The api key associated with the account from which to pull data.
    :type api_key: str
    """
    import stripe
    stripe.api_key = api_key

    account = stripe.Account.retrieve()
    supported_payment_currencies = stripe.CountrySpec.retrieve(account["country"])["supported_payment_currencies"]

    return [(currency, currency.upper()) for currency in supported_payment_currencies]


def dict_nested_accessor(d, name):
    """
    Access a dictionary value, possibly in a nested dictionary.

    >>> dict_nested_accessor({'id': 'joe'}, 'id')
    "joe"
    >>> dict_nested_accessor({'inner': {'id': 'joe'}}, 'inner.id')
    "joe"

    :type d: dict
    """
    names = name.split(".", 1)
    if len(names) > 1:
        return dict_nested_accessor(d[names[0]], names[1])
    else:
        return d[name]


def convert_tstamp(response, field_name=None):
    """
    Convert a Stripe API timestamp response (unix epoch) to a native datetime.

    :rtype: datetime
    """
    # Overrides the set timezone to UTC - I think...
    tz = timezone.utc if settings.USE_TZ else None

    if not field_name:
        return datetime.datetime.fromtimestamp(response, tz)
    else:
        if field_name in response and response[field_name]:
            return datetime.datetime.fromtimestamp(response[field_name], tz)



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.signals.

   :synopsis: dj-stripe - signals are sent for each event Stripe sends to the app

.. moduleauthor:: Daniel Greenfeld (@pydanny)
.. moduleauthor:: Alex Kavanaugh (@kavdev)
.. moduleauthor:: Lee Skillen (@lskillen)

Stripe docs for Webhooks: https://stripe.com/docs/webhooks
"""

from django.core.exceptions import ObjectDoesNotExist
from django.dispatch import Signal, receiver
from django.db.models.signals import pre_delete
from . import settings as djstripe_settings


webhook_processing_error = Signal(providing_args=["data", "exception"])

# A signal for each Event type. See https://stripe.com/docs/api#event_types

WEBHOOK_SIGNALS = dict([
    (hook, Signal(providing_args=["event"]))
    for hook in [
        "account.updated",
        "account.application.deauthorized",
        "account.external_account.created",
        "account.external_account.deleted",
        "account.external_account.updated",
        "application_fee.created",
        "application_fee.refunded",
        "application_fee.refund.updated",
        "balance.available",
        "bitcoin.receiver.created",
        "bitcoin.receiver.filled",
        "bitcoin.receiver.updated",
        "bitcoin.receiver.transaction.created",
        "charge.captured",
        "charge.failed",
        "charge.refunded",
        "charge.succeeded",
        "charge.updated",
        "charge.dispute.closed",
        "charge.dispute.created",
        "charge.dispute.funds_reinstated",
        "charge.dispute.funds_withdrawn",
        "charge.dispute.updated",
        "coupon.created",
        "coupon.deleted",
        "coupon.updated",
        "customer.created",
        "customer.deleted",
        "customer.updated",
        "customer.discount.created",
        "customer.discount.deleted",
        "customer.discount.updated",
        "customer.source.created",
        "customer.source.deleted",
        "customer.source.updated",
        "customer.subscription.created",
        "customer.subscription.deleted",
        "customer.subscription.trial_will_end",
        "customer.subscription.updated",
        "invoice.created",
        "invoice.payment_failed",
        "invoice.payment_succeeded",
        "invoice.updated",
        "invoiceitem.created",
        "invoiceitem.deleted",
        "invoiceitem.updated",
        "plan.created",
        "plan.deleted",
        "plan.updated",
        "transfer.created",
        "transfer.failed",
        "transfer.paid",
        "transfer.reversed",
        "transfer.updated",
        "ping"
    ]
])


@receiver(pre_delete, sender=djstripe_settings.get_subscriber_model_string())
def on_delete_subscriber_purge_customer(instance=None, **kwargs):
    """ Purge associated customers when the subscriber is deleted. """
    try:
        customer = instance.customer
    except (AttributeError, ObjectDoesNotExist):
        return

    customer.purge()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-12-29 00:41
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations
import djstripe.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0019_purge_on_subscriber_delete'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='webhook_message',
            field=djstripe.fields.StripeJSONField(help_text='data received at webhook. data should be considered to be garbage until validity check is run and valid flag is set'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='application_fee_percent',
            field=djstripe.fields.StripePercentField(decimal_places=2, help_text='A positive decimal that represents the fee percentage of the subscription invoice amount that will be transferred to the application owner’s Stripe account each billing period.', max_digits=5, null=True, validators=[django.core.validators.MinValueValidator(1.0), django.core.validators.MaxValueValidator(100.0)]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-06-25 17:51
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0014_auto_20160625_1851'),
    ]

    operations = [
        migrations.CreateModel(
            name='UpcomingInvoice',
            fields=[
                ('invoice_ptr', models.OneToOneField(
                    auto_created=True,
                    on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True,
                    serialize=False, to='djstripe.Invoice')),
            ],
            options={
                'abstract': False,
            },
            bases=('djstripe.invoice',),
        ),
        migrations.AlterField(
            model_name='invoiceitem',
            name='invoice',
            field=models.ForeignKey(help_text='The invoice to which this invoiceitem is attached.',
                                    null=True, on_delete=django.db.models.deletion.CASCADE,
                                    related_name='invoiceitems', to='djstripe.Invoice'),
        ),
    ]



#!/usr/bin/env python

import ast
import os
import sys


class MetadataFinder(ast.NodeVisitor):
    def __init__(self):
        self.version = None
        self.summary = None
        self.author = None
        self.email = None
        self.uri = None
        self.licence = None

    def visit_Assign(self, node):
        if node.targets[0].id == '__version__':
            self.version = node.value.s
        elif node.targets[0].id == '__summary__':
            self.summary = node.value.s
        elif node.targets[0].id == '__author__':
            self.author = node.value.s
        elif node.targets[0].id == '__email__':
            self.email = node.value.s
        elif node.targets[0].id == '__uri__':
            self.uri = node.value.s
        elif node.targets[0].id == '__license__':
            self.license = node.value.s


with open(os.path.join('djstripe', '__init__.py')) as open_file:
    finder = MetadataFinder()
    finder.visit(ast.parse(open_file.read()))

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload --sign')
    os.system('python setup.py bdist_wheel upload --sign')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on github:")
    os.system("git tag -a %s -m 'version %s'" % (finder.version, finder.version))
    os.system("git push --tags")
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

INSTALL_REQUIRES = [
    'django!=1.9,>=1.8',  # Django has a regression in 1.9, see #275
    'django-braces>=1.9.0',
    'django-model-utils>=2.5.2',
    'django-polymorphic>=1.0',
    'jsonfield>=1.0.3',
    'pytz>=2016.6.1',
    'stripe>=1.41.1',
    'tqdm>=4.8.4',
    'python-doc-inherit~=0.3.0',
    'mock-django~=0.6.10',
]

setup(
    name='dj-stripe-kando',
    version=finder.version,
    description=finder.summary,
    long_description=readme + '\n\n' + history,
    author=finder.author,
    author_email=finder.email,
    url=finder.uri,
    packages=[
        'djstripe',
    ],
    package_dir={'djstripe': 'djstripe'},
    include_package_data=True,
    install_requires=INSTALL_REQUIRES,
    license=finder.license,
    zip_safe=False,
    keywords='stripe django',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-22 14:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


DJSTRIPE_SUBSCRIBER_MODEL = getattr(settings, "DJSTRIPE_SUBSCRIBER_MODEL", settings.AUTH_USER_MODEL)


class Migration(migrations.Migration):

    dependencies = [
        ('djstripe', '0021_auto_20170107_0813'),
    ]

    operations = [
        migrations.AlterField(
            model_name='customer',
            name='subscriber',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, to=DJSTRIPE_SUBSCRIBER_MODEL, related_name="customer"),
        ),
    ]



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.management.commands.djstripe_init_plans.

   :synopsis: dj-stripe - init_plans command.

.. moduleauthor:: @kavdev, @pydanny

"""
from __future__ import unicode_literals

from django.core.management.base import BaseCommand

from ...sync import sync_plans


class Command(BaseCommand):
    """Make sure your Stripe account has the plans."""

    help = "Make sure your Stripe account has the plans"

    def handle(self, *args, **options):
        """Call sync_plans."""
        sync_plans()



# -*- coding: utf-8 -*-
"""
.. module:: dj-stripe.contrib.rest_framework.views.

    :synopsis: Views for the dj-stripe REST API.

.. moduleauthor:: Philippe Luickx (@philippeluickx)

"""

from __future__ import unicode_literals

from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from ...models import Customer
from ...settings import subscriber_request_callback, CANCELLATION_AT_PERIOD_END
from .serializers import SubscriptionSerializer, CreateSubscriptionSerializer


class SubscriptionRestView(APIView):
    """API Endpoints for the Subscription object."""

    permission_classes = (IsAuthenticated,)

    def get(self, request, **kwargs):
        """
        Return the customer's valid subscriptions.

        Returns with status code 200.
        """
        try:
            customer, _created = Customer.get_or_create(subscriber=subscriber_request_callback(self.request))

            serializer = SubscriptionSerializer(customer.subscription)
            return Response(serializer.data)
        except:
            return Response(status=status.HTTP_204_NO_CONTENT)

    def post(self, request, **kwargs):
        """
        Create a new current subscription for the user.

        Returns with status code 201.
        """
        serializer = CreateSubscriptionSerializer(data=request.data)

        if serializer.is_valid():
            try:
                customer, _created = Customer.get_or_create(
                    subscriber=subscriber_request_callback(self.request)
                )
                customer.add_card(serializer.data["stripe_token"])
                customer.subscribe(
                    serializer.data["plan"],
                    serializer.data.get("charge_immediately", True)
                )
                return Response(serializer.data, status=status.HTTP_201_CREATED)
            except:
                # TODO: Better error messages
                return Response(
                    "Something went wrong processing the payment.",
                    status=status.HTTP_400_BAD_REQUEST
                )

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, **kwargs):
        """
        Mark the customers current subscription as cancelled.

        Returns with status code 204.
        """
        try:
            customer, _created = Customer.get_or_create(subscriber=subscriber_request_callback(self.request))
            customer.subscription.cancel(at_period_end=CANCELLATION_AT_PERIOD_END)

            return Response(status=status.HTTP_204_NO_CONTENT)
        except:
            return Response("Something went wrong cancelling the subscription.", status=status.HTTP_400_BAD_REQUEST)



# -*- coding: utf-8 -*-
"""
.. module:: djstripe.webhooks.

  :synopsis: dj-stripe - Views related to the djstripe app.

.. moduleauthor:: @kavdev, @pydanny, @lskillen, @wahuneke, @dollydagr, @chrissmejia
"""
from __future__ import unicode_literals

import json
import logging

from braces.views import CsrfExemptMixin, FormValidMessageMixin, LoginRequiredMixin, SelectRelatedMixin
from django.contrib import messages
from django.contrib.auth import logout as auth_logout, REDIRECT_FIELD_NAME
from django.core.urlresolvers import reverse_lazy, reverse
from django.http import HttpResponse
from django.http.response import HttpResponseNotFound
from django.shortcuts import render, redirect
from django.utils.encoding import smart_str
from django.utils.http import is_safe_url
from django.views.generic import DetailView, FormView, TemplateView, View
from stripe.error import StripeError

from . import settings as djstripe_settings
from .forms import PlanForm, CancelSubscriptionForm
from .mixins import PaymentsContextMixin, SubscriptionMixin
from .models import Customer, Event, EventProcessingException, Plan
from .sync import sync_subscriber
from .webhooks import TEST_EVENT_ID

logger = logging.getLogger(__name__)

# ============================================================================ #
#                                 Account Views                                #
# ============================================================================ #


class AccountView(LoginRequiredMixin, SelectRelatedMixin, SubscriptionMixin, PaymentsContextMixin, TemplateView):
    """Shows account details including customer and subscription details."""

    template_name = "djstripe/account.html"


# ============================================================================ #
#                                 Billing Views                                #
# ============================================================================ #

class ChangeCardView(LoginRequiredMixin, PaymentsContextMixin, DetailView):
    """TODO: Needs to be refactored to leverage forms and context data."""

    template_name = "djstripe/change_card.html"

    def get_object(self):
        """
        Return a Customer object.

        Ether returns the Customer object from the current class instance or
        uses get_or_create.
        """
        if hasattr(self, "customer"):
            return self.customer
        self.customer, _created = Customer.get_or_create(
            subscriber=djstripe_settings.subscriber_request_callback(self.request)
        )
        return self.customer

    def post(self, request, *args, **kwargs):
        """TODO: Raise a validation error when a stripe token isn't passed. Should be resolved when a form is used."""
        customer = self.get_object()
        try:
            send_invoice = not customer.default_source
            customer.add_card(
                request.POST.get("stripe_token")
            )
            if send_invoice:
                customer.send_invoice()
            customer.retry_unpaid_invoices()
        except StripeError as exc:
            messages.info(request, "Stripe Error")
            return render(
                request,
                self.template_name,
                {
                    "customer": self.get_object(),
                    "stripe_error": str(exc)
                }
            )
        messages.info(request, "Your card is now updated.")
        return redirect(self.get_post_success_url())

    def get_post_success_url(self):
        """Make it easier to do custom dj-stripe integrations."""
        return reverse("djstripe:account")


class HistoryView(LoginRequiredMixin, SelectRelatedMixin, DetailView):
    """A view used to return customer history of invoices."""

    template_name = "djstripe/history.html"
    model = Customer
    select_related = ["invoice"]

    def get_object(self):
        """Return a Customer object."""
        customer, _created = Customer.get_or_create(
            subscriber=djstripe_settings.subscriber_request_callback(self.request)
        )
        return customer


class SyncHistoryView(CsrfExemptMixin, LoginRequiredMixin, View):
    """TODO: Needs to be refactored to leverage context data."""

    template_name = "djstripe/includes/_history_table.html"

    def post(self, request, *args, **kwargs):
        """Render the template while injecting extra context."""
        return render(
            request,
            self.template_name,
            {"customer": sync_subscriber(djstripe_settings.subscriber_request_callback(request))}
        )


# ============================================================================ #
#                              Subscription Views                              #
# ============================================================================ #

class ConfirmFormView(LoginRequiredMixin, FormValidMessageMixin, SubscriptionMixin, FormView):
    """A view used to confirm customers into a subscription plan."""

    form_class = PlanForm
    template_name = "djstripe/confirm_form.html"
    success_url = reverse_lazy("djstripe:history")
    form_valid_message = "You are now subscribed!"

    def get(self, request, *args, **kwargs):
        """Override ConfirmFormView GET to perform extra validation.

        - Returns 404 when no plan exists.
        - Redirects to djstripe:subscribe when customer is already subscribed to this plan.
        """
        plan_id = self.kwargs['plan_id']

        if not Plan.objects.filter(id=plan_id).exists():
            return HttpResponseNotFound()

        customer, _created = Customer.get_or_create(
            subscriber=djstripe_settings.subscriber_request_callback(self.request)
        )

        if (customer.subscription and str(customer.subscription.plan.id) == plan_id and
                customer.subscription.is_valid()):
            message = "You already subscribed to this plan"
            messages.info(request, message, fail_silently=True)
            return redirect("djstripe:subscribe")

        return super(ConfirmFormView, self).get(request, *args, **kwargs)

    def get_context_data(self, *args, **kwargs):
        """Return ConfirmFormView's context with plan_id."""
        context = super(ConfirmFormView, self).get_context_data(**kwargs)
        context['plan'] = Plan.objects.get(id=self.kwargs['plan_id'])
        return context

    def post(self, request, *args, **kwargs):
        """
        Handle POST requests.

        Instantiates a form instance with the passed POST variables and
        then checks for validity.
        """
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        if form.is_valid():
            try:
                customer, _created = Customer.get_or_create(
                    subscriber=djstripe_settings.subscriber_request_callback(self.request)
                )
                customer.add_card(self.request.POST.get("stripe_token"))
                customer.subscribe(form.cleaned_data["plan"])
            except StripeError as exc:
                form.add_error(None, str(exc))
                return self.form_invalid(form)
            return self.form_valid(form)
        else:
            return self.form_invalid(form)


class SubscribeView(LoginRequiredMixin, SubscriptionMixin, TemplateView):
    """A view to render the subscribe template."""

    template_name = "djstripe/subscribe.html"


class ChangePlanView(LoginRequiredMixin, FormValidMessageMixin, SubscriptionMixin, FormView):
    """
    A view used to change a Customers plan.

    TODO: Work in a trial_days kwarg.

    Also, this should be combined with ConfirmFormView.
    """

    form_class = PlanForm
    template_name = "djstripe/confirm_form.html"
    success_url = reverse_lazy("djstripe:history")
    form_valid_message = "You've just changed your plan!"

    def post(self, request, *args, **kwargs):
        """Handle a Customer changing a plan.

        Handles upgrading a plan as well. Throws an error when Customer is not subscribed to any plan.
        """
        form = PlanForm(request.POST)

        customer, _created = Customer.get_or_create(
            subscriber=djstripe_settings.subscriber_request_callback(self.request)
        )

        if not customer.subscription:
            form.add_error(None, "You must already be subscribed to a plan before you can change it.")
            return self.form_invalid(form)

        if form.is_valid():
            try:
                selected_plan = form.cleaned_data["plan"]

                # When a customer upgrades their plan, and DJSTRIPE_PRORATION_POLICY_FOR_UPGRADES is set to True,
                # we force the proration of the current plan and use it towards the upgraded plan,
                # no matter what DJSTRIPE_PRORATION_POLICY is set to.
                if djstripe_settings.PRORATION_POLICY_FOR_UPGRADES:
                    # Is it an upgrade?
                    if selected_plan.amount > customer.subscription.plan.amount:
                        customer.subscription.update(plan=selected_plan, prorate=True)
                    else:
                        customer.subscription.update(plan=selected_plan)
                else:
                    customer.subscription.update(plan=selected_plan)
            except StripeError as exc:
                form.add_error(None, str(exc))
                return self.form_invalid(form)
            return self.form_valid(form)
        else:
            return self.form_invalid(form)


class CancelSubscriptionView(LoginRequiredMixin, SubscriptionMixin, FormView):
    """A view used to cancel a Customer's subscription."""

    template_name = "djstripe/cancel_subscription.html"
    form_class = CancelSubscriptionForm
    success_url = reverse_lazy("djstripe:account")
    redirect_url = reverse_lazy("home")

    # messages
    subscription_cancel_message = "Your subscription is now cancelled."
    subscription_status_message = "Your subscription status is now '{status}' until '{period_end}'"

    def get_redirect_url(self):
        """
        Return the URL to redirect to when canceling is successful.
        Looks in query string for ?next, ensuring it is on the same domain.
        """
        next = self.request.GET.get(REDIRECT_FIELD_NAME)

        # is_safe_url() will ensure we don't redirect to another domain
        if next and is_safe_url(next):
            return next
        else:
            return self.redirect_url

    def form_valid(self, form):
        """Handle canceling the Customer's subscription."""
        customer, _created = Customer.get_or_create(
            subscriber=djstripe_settings.subscriber_request_callback(self.request)
        )
        subscription = customer.subscription.cancel()

        if subscription.status == subscription.STATUS_CANCELED:
            return self.status_cancel()
        else:
            # If pro-rate, they get some time to stay.
            messages.info(self.request, self.subscription_status_message.format(
                status=subscription.status, period_end=subscription.current_period_end)
            )

        return super(CancelSubscriptionView, self).form_valid(form)

    def status_cancel(self):
        """Triggered when the subscription is immediately canceled (not pro-rated)"""
        # If no pro-rate, they get kicked right out.
        messages.info(self.request, self.subscription_cancel_message)
        # logout the user
        auth_logout(self.request)
        # Redirect to next url
        return redirect(self.get_redirect_url())


# ============================================================================ #
#                                 Web Services                                 #
# ============================================================================ #


class WebHook(CsrfExemptMixin, View):
    """A view used to handle webhooks."""

    def post(self, request, *args, **kwargs):
        """
        Create an Event object based on request data.

        Creates an EventProcessingException if the webhook Event is a duplicate.
        """
        body = smart_str(request.body)
        data = json.loads(body)

        if data['id'] == TEST_EVENT_ID:
            logger.info("Test webhook received: {}".format(data['type']))
            return HttpResponse()

        if Event.stripe_objects.exists_by_json(data):
            EventProcessingException.objects.create(
                data=data,
                message="Duplicate event record",
                traceback=""
            )
        else:
            event = Event._create_from_stripe_object(data)
            event.validate()

            if djstripe_settings.WEBHOOK_EVENT_CALLBACK:
                djstripe_settings.WEBHOOK_EVENT_CALLBACK(event)
            else:
                event.process()

        return HttpResponse()



# -*- coding: utf-8 -*-
"""
.. module:: dj-stripe.contrib.rest_framework.permissions.

    :synopsis: dj-stripe - Permissions to be used with the dj-stripe REST API.

.. moduleauthor:: @kavdev, @pydanny

"""
from rest_framework.permissions import BasePermission

from ...utils import subscriber_has_active_subscription
from ...settings import subscriber_request_callback


class DJStripeSubscriptionPermission(BasePermission):
    """A permission to be used when wanting to permit users with active subscriptions."""

    def has_permission(self, request, view):
        """
        Check if the subscriber has an active subscription.

        Returns false if:
            * a subscriber isn't passed through the request

        See ``utils.subscriber_has_active_subscription`` for more rules.

        """
        try:
            subscriber_has_active_subscription(subscriber_request_callback(request))
        except AttributeError:
            return False
